<?php
/**
 * Copyright © 2015 FiloBlu . All rights reserved.
 */
namespace FiloBlu\Flow\Helper;

class Price extends \Magento\Framework\App\Helper\AbstractHelper{

    const TYPE_PIENO = 'P';
    const TYPE_SPECIALE = 'S';

    CONST INSERT = 1;
    CONST UPDATE = 2;
    CONST DELETE = 3;


    public function getWebsitesByCountryAndCurrency($countryCode, $currencyCode) {
        $results = array();
        if (strtolower($countryCode) == Mage_Core_Model_Store::ADMIN_CODE) {
            //$website = Mage::getModel('core/website')->load(Mage_Core_Model_Store::ADMIN_CODE);
            $website = Mage::app()->getWebsite(Mage_Core_Model_Store::ADMIN_CODE);
            $websiteCurrencyCode = strtolower($website->getConfig('currency/options/default'));
            if (strtolower($currencyCode) == strtolower($websiteCurrencyCode)) {
                return array($website);
            }
        }

        /*
         * default store should not be considered here, it is handled above
         * otherwise we can update prices on default store when sending IT/EUR, if default store has IT/EUR as countryu and currency values
         */
        $websites = Mage::app()->getWebsites($withDefault = false);
        foreach ($websites as $website) {
            if (!empty($countryCode)) {
                //TODO mettere un parametro di configurazione se settare i prezzi per Website code o country code
                //$websiteCountryCode = strtolower($website->getConfig('general/country/default'));
                $websiteCountryCode = strtolower($website->getCode());
                if (strtolower($countryCode) != $websiteCountryCode) {
                    continue;
                }
            }

            //WHLP-834
            $websiteCurrencyCode = strtolower($website->getConfig('currency/options/default'));
            if (strtolower($currencyCode) != strtolower($websiteCurrencyCode)) {
                continue;
            }

            $results[] = $website;
        }

        return $results;
    }

    public function getPriceTypes() {
        return array(
            self::TYPE_PIENO,
            self::TYPE_SPECIALE
        );
    }

    public function getMappedTierPriceInfo($priceSetCode) {
        $mapping = Mage::helper('fbflow')->toArray(Mage::getStoreConfig('fbflow/pri/tier_price_info_mapping'));
        if (isset($mapping[$priceSetCode])) {
            return $mapping[$priceSetCode];
        }

        return false;
    }


    /**
     *
     * salva il group price di un configurabili dati i group price minimi dei figli
     * @param $configurableId
     * @param $websiteId
     * @param $fatherGroupPrices array group price minimi dei figli
     */
    static public function saveConfigurableGroupPrice($configurableId,$websiteId,$fatherGroupPrices){
        $resource = Mage::getResourceModel('catalog/product_attribute_backend_groupprice');
        $oldPrices = $resource->loadPriceData($configurableId, $websiteId);
        foreach ($fatherGroupPrices as $wg) {
            foreach ($wg as $fgp) {
                $action = self::checkConfigurableOldPrices($oldPrices, $fgp);
                $fprice = new Varien_Object($fgp);
                switch ($action) {
                    case self::INSERT:
                    case self::UPDATE:
                        $fprice->setEntityId($configurableId);
                        $resource->savePriceData($fprice);
                        break;
                    case self::DELETE:
                        if (isset($fgp['value_id'])) {
                            $resource->deletePriceData($configurableId, null, $fgp['value_id']);
                        }
                        break;
                }
            }
        }
    }

    public function getMappedGroupPriceInfo($priceSetCode) {
        $mapping = Mage::helper('fbflow')->toArray(Mage::getStoreConfig('fbflow/pri/group_price_info_mapping'));
        if (isset($mapping[$priceSetCode])) {
            return array_pop($mapping[$priceSetCode]);
        }
        return false;
    }

    public function getAppliedPriceCatalog($orderItem) {
        return "";
    }

    public function getCheckOnlyEnabledChildren(){
        return (bool)Mage::getStoreConfig('fbflow/pri/check_only_enabled_children');
    }

    public function getLowerChildProductGroupPrice($configurableId, $storeId, $websiteId)
    {

        if(version_compare(Mage::getVersion(), '1.7.0.0', '<')){
            return array();
        }

        $master_group_price = array();

        $configurable = Mage::getModel('catalog/product')->setStoreId($storeId)->load($configurableId);
        if (!Mage::helper('fbflow/product')->isConfigurableItem($configurable->getTypeId())) {
            $message = sprintf("Product type is not configurable for pid #%s.", $configurableId);
            throw new \Exception($message);
        }

        $childIds = $configurable->getTypeInstance()->getUsedProductIds();
        if (count($childIds) == 0) {
            $message = sprintf("Pid #%s has no children products associated.", $configurableId);
            throw new \Exception($message);
        }

        foreach($childIds as $childId) {
            $group_price[$childId] = Mage::getResourceModel('catalog/product_attribute_backend_groupprice')->loadPriceData($childId,$websiteId);
            foreach($group_price[$childId] as $gp){
                $master_group_price[$websiteId][$gp['cust_group']] = array('website_id' =>  $websiteId, 'all_groups' => 0, 'customer_group_id' => $gp['cust_group'], 'value' => 0 );
            }
        }

        foreach($group_price as $cgp){
            foreach($cgp as $gp) {
                if (isset($master_group_price[$gp['website_id']][$gp['cust_group']])) {
                    $master_price = $master_group_price[$gp['website_id']][$gp['cust_group']]['value'];
                    $child_price = $gp['price'];
                    if ($master_price == 0 || ($master_price > $child_price && $child_price != 0)) {
                        $master_group_price[$gp['website_id']][$gp['cust_group']]['value'] = $child_price;
                    }
                }
            }
        }

        return $master_group_price;
    }


    /**
     *  verifico se è già presente un prezzo di gruppo per il prodotto e eventualmente aggiorno o cancello
     * @param $oldPrices
     * @param $fgp
     * @return int|null
     */
    static public function checkConfigurableOldPrices($oldPrices, &$fgp)
    {
        $action = self::INSERT;
        foreach ($oldPrices as $oldP) {
            if ($oldP['cust_group'] == $fgp['customer_group_id'] && $oldP['website_id'] == $fgp['website_id']) {
                $action = null;
                $fgp['value_id'] = $oldP['price_id'];
                if ($oldP['price'] != $fgp['value']) {
                    $action = self::UPDATE;
                    $fgp['value_id'] = $oldP['price_id'];
                }
                if ($fgp['value'] == 0) {
                    $action == self::DELETE;
                    $fgp['value_id'] = $oldP['price_id'];
                }
            }
        }
        return $action;
    }


    public function getLowerChildProductPrice($configurableId, $storeId, &$childrenHaveDifferentPrices) {
        $configurable = Mage::getModel('catalog/product')->setStoreId($storeId)->load($configurableId);
        if (!Mage::helper('fbflow/product')->isConfigurableItem($configurable->getTypeId())) {
            $message = sprintf("Product type is not configurable for pid #%s.", $configurableId);
            throw new \Exception($message);
        }

        $childIds = $configurable->getTypeInstance()->getUsedProductIds();
        if (count($childIds) == 0) {
            $message = sprintf("Pid #%s has no children products associated.", $configurableId);
            throw new \Exception($message);
        }

        $specialPrice = null;
        $fullPrice = null;
        $specialPrices = array();
        $fullPrices = array();
        $childrenHaveDifferentPrices = false;
        //Mage::log('Parent id: '.$configurableId);
        foreach ($childIds as $childId) {
            $product = Mage::getModel('catalog/product')->setStoreId($storeId)->load($childId);
            if($this->getCheckOnlyEnabledChildren()){
                if($product->getStatus() == Mage_Catalog_Model_Product_Status::STATUS_DISABLED){
                    continue;
                }
            }

            if (($storeId == Mage_Core_Model_App::ADMIN_STORE_ID || $product->getExistsStoreValueFlag(FiloBlu_Flow_Model_From_Price::SPECIAL_PRICE_ATTRIBUTE_CODE)) && $product->getSpecialPrice() > 0) {
                if (!empty($specialPrices) && !in_array($product->getSpecialPrice(), $specialPrices)) {
                    $childrenHaveDifferentPrices = true;
                }
                $specialPrices[$product->getId()] = floatval($product->getSpecialPrice());
                //Mage::log('Setting special for '.$childId);
            }

            if (($storeId == Mage_Core_Model_App::ADMIN_STORE_ID || $product->getExistsStoreValueFlag(FiloBlu_Flow_Model_From_Price::PRICE_ATTRIBUTE_CODE)) && $product->getPrice() > 0) {
                if (!empty($fullPrices) && !in_array($product->getPrice(), $fullPrices)) {
                    $childrenHaveDifferentPrices = true;
                }
                $fullPrices[$product->getId()] = floatval($product->getPrice());
                //Mage::log('Setting full for '.$childId);
            }

            unset($product);
        }

        //Mage::log('Special prices: '.print_r($specialPrices, 1));
        //Mage::log('Full prices: '.print_r($fullPrices, 1));

        if (!empty($specialPrices)) {
            asort($specialPrices);
            $specialPrice = reset($specialPrices);
            $productId = key($specialPrices);
            $fullPrice = $fullPrices[$productId];
            if (empty($fullPrice) && !Mage::helper('fbflow/price')->allowSpecialPriceOnly()) {
                $message = sprintf("Pid #%s has no full price, while having the cheapest special price, if you want to process it anyway please enable the related configuration setting", $productId);
                throw new \Exception($message);
            }
        } elseif (!empty($fullPrices)) {
            asort($fullPrices);
            $fullPrice = reset($fullPrices);
        }
        //Mage::log('Price: '.print_r($fullPrice, 1));
        //Mage::log('Special price: '.print_r($specialPrice, 1));
        return array("price" => $fullPrice, "special_price" => $specialPrice);
    }

    /**
     * get fixed price for bundle with dyn. prices only if
     * - all options has ONLY one selection
     * - defined as default
     * - with default qty
     * - and qty NOT editable
     *
     * @param Mage_Catalog_Model_Product $product
     *
     * @return string
     */
    public function getBundleFinalPrice($product) {
        if ($product->getTypeId() != Mage_Catalog_Model_Product_Type::TYPE_BUNDLE) {
            $message = sprintf("Product with sku '%s' is not a bundle.", $product->getSku());
            throw new \Exception($message);
        }

        if ($product->getPriceType() == Mage_Bundle_Model_Product_Price::PRICE_TYPE_FIXED) {
            return $product->getFinalPrice();
        }

        //read bundle options
        $typeInstance = $product->getTypeInstance(true);
        $typeInstance->setStoreFilter($product->getStoreId(), $product);

        $optionCollection = $typeInstance->getOptionsCollection($product);

        $selectionCollection = $typeInstance->getSelectionsCollection(
            $typeInstance->getOptionsIds($product), $product
        );

        //ignore sealability to determine price
        $options = $optionCollection->appendSelections($selectionCollection, false, true
        );


        $price = 0;
        $priceModel = $product->getPriceModel();
        foreach ($options as $option) {

            $selection = $option->getDefaultSelection();

            if ($selection == null) {
                $message = sprintf("No default selection defined for option '%s' of bundle with sku '%s'.", $option->getTitle(), $product->getSku());
                throw new \Exception($message);
            }


            $qtyIsLocked = $selection->getSelectionQty() && !$selection->getSelectionCanChangeQty();


            if (!$qtyIsLocked) {
                $message = sprintf("Selection product with id #%s of bundle with sku '%s' must have qty locked to 1 and not editable.", $selection->getProductId(), $product->getSku());
                throw new \Exception($message);
            }

            $price += $priceModel->getSelectionPreFinalPrice($product, $selection, $selection->getSelectionQty());
        }

        return $price;
    }

    public function allowSpecialPriceOnly() {
        return (bool) Mage::getStoreConfig('fbflow/pri/allow_special_price_only');
    }

    public function useGlobalToBaseRate() {
        return (bool) Mage::getStoreConfig('fbflow/tsv/order/use_global_to_base_rate');
    }

    public function getCurrencyRate($fromCurrency,$toCurrency){
        if(!is_a( $fromCurrency,'Mage_Directory_Model_Currency') ){
            if(empty($fromCurrency) || !is_string($fromCurrency)){
                $message = sprintf("Invalid from currency : %s",$fromCurrency);
                throw new \Exception($message);
            }
            $fromCurrency = Mage::getModel('directory/currency')->load($fromCurrency);
        }
        //toCurrency get validated by getRate
        return $fromCurrency->getRate($toCurrency);
    }

    public function isBulkImport(){
        return (bool) Mage::getStoreConfig('fbflow/pri/bulk_import');
    }

    public function getBulkSize(){
        return Mage::getStoreConfig('fbflow/pri/bulk_size');
    }
}

//End of class