<?php
/**
 * Copyright © 2015 FiloBlu. All rights reserved.
 */

namespace FiloBlu\Flow\Setup;

use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

/**
 * @codeCoverageIgnore
 */
class InstallSchema implements InstallSchemaInterface
{
    /**
     * {@inheritdoc}
     */
    public function install(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
	
        $installer = $setup;

        $installer->startSetup();

	/**
         * Create table 'flow_inboundflow' if not exists
         */
        if (!$installer->tableExists('flow_inboundflow')) {

            $table = $installer->getConnection()->newTable(
                $installer->getTable('flow_inboundflow')
            )
                ->addColumn(
                    'id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                    'flow_inboundflow'
                )
                ->addColumn(
                    'name',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    'name'
                )
                ->addColumn(
                    'flow',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    'flow'
                )
                ->addColumn(
                    'type',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    'type'
                )
                ->addColumn(
                    'channel',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    'channel'
                )
                ->addColumn(
                    'priority',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    'priority'
                )
                ->addColumn(
                    'status',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    'status'
                )
                ->addColumn(
                    'retry',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    'retry'
                )
                ->addColumn(
                    'receiving_time',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                    null,
                    ['nullable' => false],
                    'receiving_time'
                )
                ->addColumn(
                    'log',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    'log'
                )
                ->addColumn(
                    'comment',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    'comment'
                )
                /* ->addIndex(
                    $installer->getIdxName(
                        'FILENAME',
                        [
                            'name'
                        ],
                        AdapterInterface::INDEX_TYPE_UNIQUE 
                    ),
                    [
                        'name'
                    ] ,
                    [
                        'type' => AdapterInterface::INDEX_TYPE_UNIQUE
                    ] 
                ) */
                ->setComment(
                    'FiloBlu Flow flow_inboundflow'
                );

            $installer->getConnection()->createTable($table);
        }
        
        
        /**
         * Create table 'flow_channels' if not exists
         */
        
        if (!$installer->tableExists('flow_channels')) {

            $table_channels = $installer->getConnection()->newTable(
                $installer->getTable('flow_channels')
            )
                ->addColumn(
                    'id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                    'flow_channels'
                )
                ->addColumn(
                    'name',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '40',
                    [],
                    'name'
                )
                ->addColumn(
                    'data',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '64k',
                    [],
                    'data'
                )
                ->addColumn(
                    'type',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '20',
                    [],
                    'type'
                )


                ->setComment(
                    'FiloBlu Flow flow_channels'
                );

            $installer->getConnection()->createTable($table_channels);


        }
        
        
         /**
         * Create table 'flow_from_ite' if not exists
         */
        if (!$installer->tableExists('flow_from_ite')) {

            $table_ite = $installer->getConnection()->newTable(
                $installer->getTable('flow_from_ite')
            )

                ->addColumn(
                    'codice_articolo',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '100',
                    [],
                    'sku'
                )
                ->addColumn(
                    'codice_articolo_del_fornitore',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '100',
                    [],
                    'barcode'
                )
                ->addColumn(
                    'nome_articolo',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '100',
                    [],
                    'nome_articolo'
                )
                ->addColumn(
                    'codice_configurabile',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '30',
                    [],
                    'codice_configurabile'
                )
                ->addColumn(
                    'fascia_iva_articoli',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '10',
                    [],
                    'fascia_iva_articoli'
                )
                ->addColumn(
                    'meta_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                    'flow_channels'
                )
                ->addColumn(
                    'meta_ref_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    '11',
                    [],
                    'meta_ref_id'
                )
                ->addColumn(
                    'meta_file',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    ['unsigned' => true, 'nullable' => false],
                    'meta_file'
                )

                ->addColumn(
                    'meta_processed',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    '1',
                    [],
                    'meta_processed'
                )
                ->addColumn(
                    'meta_insert_time',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    '10',
                    ['unsigned' => true, 'nullable' => false],
                    'meta_insert_time'
                )

                ->addColumn(
                    'meta_process_time',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    '10',
                    [],
                    'meta_process_time'
                )

                ->addIndex(
                    $installer->getIdxName('flow_from_ite_meta_file', ['meta_file']),
                    ['meta_file']
                )

                ->addForeignKey(
                    $installer->getFkName(
                        'flow_from_ite',
                        'meta_file',
                        'flow_inboundflow',
                        'id'
                    ),
                    'meta_file',
                    $installer->getTable('flow_inboundflow'),
                    'id',
                    \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE
                )

                /*{{CedAddTableColumn}}}*/


                ->setComment(
                    'FiloBlu Flow flow_channels'
                );

            $installer->getConnection()->createTable($table_ite);
            /*{{CedAddTable}}*/

        }
        
        
        /**
         * Create table 'flow_from_eavs' if not exists
         */
        if (!$installer->tableExists('flow_from_eavs')) {

            $table_eavs = $installer->getConnection()->newTable(
                $installer->getTable('flow_from_eavs')
            )

                ->addColumn(
                    'e',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '255',
                    [],
                    'e'
                )
                ->addColumn(
                    'a',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '255',
                    [],
                    'a'
                )
                ->addColumn(
                    'v',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '65535',
                    [],
                    'v'
                )
                ->addColumn(
                    's',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '255',
                    [],
                    's'
                )
                ->addColumn(
                    'meta_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                    'flow_channels'
                )
                ->addColumn(
                    'meta_ref_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    '11',
                    [],
                    'meta_ref_id'
                )
                ->addColumn(
                    'meta_file',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [],
                    'meta_file'
                )

                ->addColumn(
                    'meta_processed',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    '1',
                    [],
                    'meta_processed'
                )
                ->addColumn(
                    'meta_insert_time',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    '10',
                    ['unsigned' => true, 'nullable' => false],
                    'meta_insert_time'
                )

                ->addColumn(
                    'meta_process_time',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    '10',
                    [],
                    'meta_process_time'
                );                ;
        $installer->getConnection()->createTable($table_eavs);

        }
        
        
        /**
         * Create table 'flow_from_price' if not exists
         */
        if (!$installer->tableExists('flow_from_price')) {

            $table_price = $installer->getConnection()->newTable(
                $installer->getTable('flow_from_price')
            )
                ->addColumn(
                    'sku',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '60',
                    [],
                    'sku'
                )

                ->addColumn(
                    'paese',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '20',
                    [],
                    'paese'
                )

                ->addColumn(
                    'prezzo',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
                    '12,4',
                    ['nullable' => false, 'default' => '0.0000'],
                    'prezzo'
                )

                ->addColumn(
                    'valuta',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    '3',
                    [],
                    'valuta'
                )
                ->addColumn(
                    'meta_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    ['identity' => true, 'unsigned' => true, 'nullable' => false, 'primary' => true],
                    'flow_channels'
                )
                ->addColumn(
                    'meta_ref_id',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    '11',
                    [],
                    'meta_ref_id'
                )
                ->addColumn(
                    'meta_file',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [],
                    'meta_file'
                )

                ->addColumn(
                    'meta_processed',
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    '1',
                    [],
                    'meta_processed'
                )
                ->addColumn(
                    'meta_insert_time',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    '10',
                    ['unsigned' => true, 'nullable' => false],
                    'meta_insert_time'
                )

                ->addColumn(
                    'meta_process_time',
                    \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    '10',
                    [],
                    'meta_process_time'
                );

            $installer->getConnection()->createTable($table_price);
        }




        $installer->endSetup();

    }
}